      PROGRAM STAYNL
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C
C     THE STAYNL IS AN EXTENDED VERSION OF THE PROGRAM STAY'SL WRITTEN
C     BY F. G. PEREY.  IT HAS BEEN DEVELOPED BY A TEAM OF THE ECN,
C     NETHERLANDS ENERGY RESEARCH FOUNDATION (PETTEN) AND THE BME-TR,
C     TRAINING NUCLEAR REACTOR (*) OF THE BUDAPEST TECHNICAL UNIVERSITY.
C     THE MEMBERS OF THE TEAM HAVE BEEN (IN ALPHABETICAL ORDER):
C                      H. J.  NOLTHENIUS (ECN)
C                      H. CH. RIEFFE     (ECN)
C                      E. J.  SZONDI     (BME-TR)
C                      W. L.  ZIJP       (ECN)
C                      E. M.  ZSOLNAY    (BME-TR)
C     (*) NEW NAME: INSTITUTE OF NUCLEAR TECHNIQUES
C
C     VERSION 78-1  (JAN. 1978)     FORTRAN-IV, BATCH PROCESSING
C                                   ON CDC-6600 OF THE STAY'SL.
C                                   NEW NAME: MAINSL.
C     VERSION 82-1  (JUNE 1982)     OPTIMAL NORMALIZATION (F0) ADDED
C                                   NEW NAME: STAYF0, CDC-6600.
C     VERSION 84-1  (MAY 1984)      CROSS-COVARIANCE (NONLINEAR) TERM
C                                   ADDED, NEW NAME: STAYNL, CDC-6600.
C     VERSION 85-1  (MAY 1985)      IBM SYSTEM/360.
C     VERSION 88-1  (OCT. 1988)     USER-FRIENDLY I/O ORGANIZATION,
C                                   EXTENDED DATA VALIDATION,
C                                   INPUT SPECTRUM SCALING,
C                                   CORRECTED STD(F0) CALCULATION.
C                                   FORTRAN-77, IBM PC/AT.
C     VERSION 90-1  (JAN. 1990)     IF THE INPUT COVARIANCE MATRIX IS
C                                   SINGULAR, THE CONSISTENCY CHECK IS
C                                   BY-PASSED; DOUBLE PRECISION
C                                   CALCULATION AGAINST UNDER/OVERFLOW
C     VERSION 91-1  (MAY 1991)      I/O FORMATS ADJUSTED TO THE NMF-90
C     VERSION 92-1  (MAY 1992)      MODIFICATIONS DUE TO THE NON-
C                                   STANDARD BEHAVIOUR OF THE MICROSOFT
C                                   FORTRAN77 COMPILER (FL 5.00)
C     VERSION 93-1  (MAY 1993)      CONSISTENCY CHECK OPTIONAL,
C                                   WEIGHT OF STD(F0) GIVEN BY USER.
C     VERSION 94-1  (SEP. 1994)     IMPROVED DIAGNOSTICS
C     VERSION 95-1  (MAY 1995)      IMPROVED TRANSPORTABILITY
C
C     PURPOSE
C     -------
C     THIS PROGRAM SOLVES THE NEUTRON SPECTRUM ADJUSTMENT TASK BY
C     LINEARIZATION. (SOME MATHEMATICAL BACKGROUND IS GIVEN EG IN
C     APP. #1 AND #4 OF THE FINAL REPORT ON THE REAL-80 EXERCISE.)
C     THE PROGRAM OUTPUTS THE SOLUTION SPECTRUM ONLY IF THE INITIAL
C     (INPUT) NEUTRON SPECTRUM LIES IN THE VICINITY OF THE TRUE
C     SPECTRUM (IE THE NECESSARY ADJUSTMENT IS SMALL), AND THE
C     INPUT COVARIANCE MATRICES ARE CORRECT, OTHERWISE THE OUTPUT
C     SPECTRUM MAY BE MEANINGLESS. (WARNING: THIS PROGRAM DOES NOT
C     CHECK WHETHER THE INPUT COVARIANCE MATRICES ARE POSITIVE
C     DEFINITE. RUN THE PROGRAM ACORNS (AVAILABLE AT THE BME-TR,
C     IAEA-NDS, AND AT ORNL-RSIC) BEFORE RUNNING THIS PROGRAM.)
C
C     IF AN ERROR OCCURS, THE ERROR CODE IS PRINTED, AS FOLLOWS:
C           STOP 1: INCONSISTENT INPUT
C           STOP 2: THE WEIGHTING MATRIX IS SINGULAR
C           STOP 3: ERRONEOUS VALUE IN THE INPUT
C           STOP 4: ERRONEOUS KEYWORD IN THE INPUT
C           STOP 5: ERRONEOUS I/O ORGANIZATION OR PREMATURE END OF DATA
C           STOP 6: INSUFFICIENT BLANK COMMON BLOCK SIZE
C
C
C     COMPUTER DEPENDENT CODING
C     =========================
C
C     THIS PROGRAM HAS BEEN WRITTEN IN STANDARD FORTRAN-77 FOR RUNNING
C     ON AN IBM PC/AT. SPECIAL REQUIREMENTS TO THE JOB OF THE X333:
C         REQUIRED MEMORY:  DEPENDS ON THE MAXIMUM PROBLEM SIZE
C                           (IF THE VALUE OF THE PARAMETERS:
C                           NGMAX=60, NRMAX=25, THEN ABOUT 330 kB FREE
C                           MEMORY IS NEEDED)
C         OPERATING SYSTEM: IBM PERSONAL COMPUTER DOS 3.30
C         COMPILER:         IBM PROFESSIONAL FORTRAN 1.00
C                           (MATH COPROCESSOR REQUIRED)
C                           OR: RYAN-McFARLAND FORTRAN 2.42
C                           OR: MICROSOFT FORTRAN 5.00
C         WORD LENGTH:      32 BITS (BOTH THE REAL AND INTEGER TYPE)
C
C     THE COMPUTER-SPECIFIC STATEMENTS ARE MARKED LIKE THE NEXT LINES:
C
C     C***** ANY EXPRESSION *****
C            ANY FORTRAN STATEMENT
C            ANY FORTRAN STATEMENT
C     C***** ANY EXPRESSION *****
C
C     FOR RUNNING THE PROGRAM ON A COMPUTER OF LONGER WORDS, FOR WHICH
C     FORTRAN-77 COMPILER IS AVAILABLE (EG CDC-175 WITH 60 BIT WORD
C     LENGTH) INSERT 'C'-S BETWEEN THE 'DOUBLE' MARKS
C
C     REFERENCE
C     ---------
C     F. G. PEREY: LEAST-SQUARES DOSIMETRY UNFOLDING: THE PROGRAM
C     STAY'SL. REPORT ORNL/TM-6062, ENDF-254. OAK RIDGE NATIONAL
C     LABORATORY, OAK RIDGE, TN, OCT. 1977.
C     W. L. ZIJP, E. M. ZSOLNAY, H. J. NOLTHENIUS, E. J. SZONDI,
C     G. C. H. M. VERHAAG, D. E. CULLEN, C. ERTEK: FINAL REPORT
C     ON THE REAL-80 EXERCISE. REPORT ECN-128, INDC(NED)-7,
C     BME-TR-RES-6/82. NETHERLANDS ENERGY RESEARCH FOUNDATION,
C     PETTEN, FEBR. 1983.
C
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INCLUDE 'STAYPARM.FOR'
C***** DOUBLE *****
      DOUBLE PRECISION A,A0,A0M,AFM,AN,ANL,AP,ASM,BS,BUFF,
     1   C,CHILIM,DA,DF,DP,DW,DWP,EL,F,FP,PF,PFP,RINF,SC,SCF,
     2   SDFR,TB,TBNL,TC,TD,TS,U,W,WP,Y
C***** DOUBLE *****
C***** SINGLE *****
C     REAL A,A0,A0M,AFM,AN,ANL,AP,ASM,BS,BUFF,
C    1   C,CHILIM,DA,DF,DP,DW,DWP,EL,F,FP,PF,PFP,RINF,SC,SCF,
C    2   SDFR,TB,TBNL,TC,TD,TS,U,W,WP,Y
C***** SINGLE *****
      REAL EN,ZS
      LOGICAL CONCHK,F0C,PRD
      INTEGER IC,IER,JER,L06,L613,NG,NG1,NGLIBR,NC,NR,NP,NS
      CHARACTER*72 TITLE
      CHARACTER*60 NAME
      DIMENSION A(NRMAX),A0(NRMAX),A0M(NRMAX2),C(NRMAX,NGMAX),
     1  F(NGMAX),DA(NRMAX),DP(NPMAX),DWP(NPMAX2),SDFR(NGMAX),
     1  U(NRMAX,NGMAX),TS(NRMAX),TB(NGMAX),AFM(NRMAX2),ASM(NRMAX2),
     2  AN(NRMAX2),
     2  W(NRMAX2),Y(NRMAX),DF(NGMAX),AP(NRMAX),SC(NRMAX),BS(NPMAX),
     3  EL(NGMAX1),EN(NGMAX1),PF(NGMAX2),
     3  FP(NGMAX),TC(NGMAX),TD(NGMAX),PFP(NGMAX),IC(NGMAX),DW(NRMAX2),
     4  NC(NPMAX),
     5  WP(NPMAX2),ZS(NGMAX),
     4  NS(NPMAX),TBNL(NGMAX),ANL(NRMAX2),BUFF(NGMAX2),CHILIM(2)
      DIMENSION NAME(NRMAX)
      IER=0
      CALL HEAD(TITLE,0,0,IER)
C-=-=-
C     CONNECT I/O FILES TO I/O UNITS
C-=-=-
      CALL CNCTIO(PRD,L06,L613,NGLIBR,IER,RINF,CONCHK)
      IF (IER.GT.0) THEN
         CALL ERRLOG(IER)
         CALL ERXPLN(IER)
      END IF
      READ (8,1,END=2,ERR=2) TITLE
      GO TO 3
    1 FORMAT (A)
    2 CALL ERRLOG(53)
      CALL ERXPLN(53)
    3 CALL HEAD(TITLE,1,L06,IER)
      READ (8,*,END=2,ERR=2) NR,NG
      IF (NG.GT.NGMAX.OR.NR.GT.NRMAX) THEN
         CALL ERRLOG(61)
         CALL ERXPLN(61)
      END IF
      IF (NG.NE.NGLIBR) THEN
         CALL ERRLOG(61)
         CALL ERXPLN(61)
      END IF
      NG1=NG+1
      NP=NR+NG
C-=-=-
C     READ THE USER'S INPUT
C-=-=-
      CALL READER(L613,NR,NG,NG1,NAME,A0,DA,A0M,EL,F,SDFR,PF,EN,IER)
      IF (IER.GT.19) THEN
         CALL HEAD(TITLE,3,6,IER)
         CLOSE (UNIT=1,STATUS='DELETE')
         CLOSE (UNIT=2,STATUS='DELETE')
         CLOSE (UNIT=3,STATUS='DELETE')
         CLOSE (UNIT=4,STATUS='KEEP')
         CLOSE (UNIT=7,STATUS='DELETE')
         CLOSE (UNIT=8,STATUS='KEEP')
         CLOSE (UNIT=9,STATUS='DELETE')
         CLOSE (UNIT=10,STATUS='DELETE')
         CLOSE (UNIT=11,STATUS='DELETE')
         CALL ERXPLN(IER)
      END IF
C-=-=-
C     UNMODIFIED USER'S DATA
C-=-=-
      CALL HEAD(TITLE,2,6,IER)
      CALL ACTCLC(NR,NG,F,C,A,ZS)
      CALL CHIDIS(NR,CHILIM)
      JER=0
      IF (CONCHK) CALL CONSIS(NR,NG,NP,A0,A0M,F,A,FP,DW,NC,NS,DP,WP,
     1   DWP,BS,CHILIM,NR,1,JER)
      IER=MAX(IER,JER)
      IF (IER.GT.19) GO TO 20
C-=-=-
C     PRE-NORMALIZATION (1,10,100,1000,1E4,1E5,&C)
C-=-=-
      CALL PRENRM(NR,NG,NAME,A0,A0M,F,C,A,SCF)
      JER=0
      IF (CONCHK) CALL CONSIS(NR,NG,NP,A0,A0M,F,A,FP,DW,NC,NS,DP,WP,
     1   DWP,BS,CHILIM,NR,2,JER)
      IER=MAX(IER,JER)
C-=-=-
C     SOLVE THE CHI2-PROBLEM FOR 'F0' NORMALIZATION FACTOR
C-=-=-
      WRITE (0,10)
   10 FORMAT (' SOLVING THE CHI2-PROBLEM FOR "F0" NORMALIZATION FACTOR')
      F0C=.TRUE.
      JER=0
      CALL STAYSL(TITLE,NR,NG,NG1,F0C,CHILIM,SCF,NAME,
     1  A0,A0M,F,C,A,DA,U,TS,TB,AFM,ASM,AN,W,Y,DF,AP,SC,BS,
     2  EL,PF,FP,TC,TD,PFP,IC,DW,NC,NS,TBNL,ANL,BUFF,ZS,JER,RINF)
      IER=MAX(IER,JER)
      IF (JER.GT.19) GO TO 20
      CALL CHIDIS(NP,CHILIM)
      IF (CONCHK) CALL CONSIS(NR,NG,NP,A0,A0M,F,A,FP,DW,NC,NS,DP,WP,
     1   DWP,BS,CHILIM,NP,3,JER)
C-=-=-
C     SOLVE THE CHI2-PROBLEM FOR THE NEUTRON FLUX DENSITY SPECTRUM
C-=-=-
      WRITE (0,15)
   15 FORMAT (' SOLVING THE CHI2-PROBLEM FOR THE NEUTRON FLUX ',
     1   'DENSITY SPECTRUM')
      F0C=.FALSE.
      JER=0
      CALL STAYSL(TITLE,NR,NG,NG1,F0C,CHILIM,SCF,NAME,
     1  A0,A0M,F,C,A,DA,U,TS,TB,AFM,ASM,AN,W,Y,DF,AP,SC,BS,
     2  EL,PF,FP,TC,TD,PFP,IC,DW,NC,NS,TBNL,ANL,BUFF,ZS,JER,RINF)
      IER=MAX(IER,JER)
      IF (JER.GT.19) GO TO 20
      JER=0
      IF (CONCHK) CALL CONSIS(NR,NG,NP,A0,A0M,F,A,FP,DW,NC,NS,DP,WP,
     1   DWP,BS,CHILIM,NP,4,JER)
      IER=MAX(IER,JER)
C-=-=-
C     END OF JOB
C-=-=-
   20 CALL HEAD(TITLE,3,6,IER)
      CLOSE (UNIT=1,STATUS='DELETE')
      CLOSE (UNIT=2,STATUS='DELETE')
      CLOSE (UNIT=3,STATUS='DELETE')
      CLOSE (UNIT=4,STATUS='KEEP')
      IF (PRD) CLOSE (UNIT=6,STATUS='KEEP')
      CLOSE (UNIT=7,STATUS='KEEP')
      CLOSE (UNIT=8,STATUS='KEEP')
      CLOSE (UNIT=9,STATUS='DELETE')
      CLOSE (UNIT=10,STATUS='DELETE')
      CLOSE (UNIT=11,STATUS='DELETE')
      CALL ERXPLN(IER)
      END
