      PROGRAM X333
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C
C     THIS PROGRAM CALCULATES GROUP CROSS SECTION VALUES AND THEIR
C     COVARIANCE MATRICES FROM THE DATA GIVEN IN FILES MF=3 AND 33 OF
C     THE IRDFxxG CROSS SECTION LIBRARIES IN ENDF-6 FORMAT. THE PROGRAM
C     HAS BEEN DEVELOPED BY A TEAM OF THE ECN, NETHERLANDS ENERGY
C     RESEARCH FOUNDATION (PETTEN) AND THE BME-TR, INSTITUTE OF NUCLEAR
C     TECHNIQUES OF THE BUDAPEST TECHNICAL UNIVERSITY. THE MEMBERS OF
C     THE TEAM HAVE BEEN (IN ALPHABETICAL ORDER):
C                      H. J.  NOLTHENIUS (ECN)
C                      H. CH. RIEFFE     (ECN)
C                      E. J.  SZONDI     (BME-TR)
C                      W. L.  ZIJP       (ECN)
C
C     VERSION 91-1 (APR. 1991) - UPDATING OF THE UNC33 CODE TO PROCESS
C                                THE IRDFxxG (xx=90 AND LATER) MASTER
C                                CROSS SECTION LIBRARIES (SEE THE
C                                REFERENCES BELOW).
C                              - CHANGE FROM SEQUENTIAL ACCESS TO THE
C                                IRDFxxG LIBRARY TO DIRECT ACCESS.
C                              - ADAPTATION OF THE STAYNL INPUT ROUTINE.
C                              - SAND II-LIKE COVER PROCESSING.
C                              - IBM PC DOS FILE NAME CONVENTION.
C
C     VERSION 92-1 (DEC. 1992) - MODIFICATIONS DUE TO THE NON-STANDARD
C                                BEHAVIOUR OF THE MICROSOFT FORTRAN 5.00
C                                COMPILER
C
C     VERSION 94-1 (AUG. 1994) - IMPROVED PROCESSING OF LB=8
C                              - SELF-/COVER-SHIELDED XSECS AREN'T KEPT
C
C     VERSION 95-1 (MAY 1995)  IMPROVED TRANSPORTABILITY
C
C     IF AN ERROR OCCURS, THE ERROR CODE IS PRINTED, AS FOLLOWS:
C           STOP 1: ERRONEOUS VALUE IN THE USER'S INPUT
C           STOP 2: ERRONEOUS STRUCTURE OF THE USER'S INPUT
C           STOP 3: LACK OF DATA IN THE MASTER CROSS SECTION LIBRARY
C           STOP 4: ERRONEOUS I/O ORGANIZATION
C           STOP 5: FATAL PROGRAM AND/OR MASTER LIBRARY ERROR
C     (DETAILS: SUBROUTINE ERXPLN)
C
C     COMPUTER DEPENDENT CODING
C     =========================
C
C     THIS PROGRAM HAS BEEN WRITTEN IN STANDARD FORTRAN-77 FOR RUNNING
C     ON AN IBM PC/AT. SPECIAL REQUIREMENTS TO THE JOB OF THE X333:
C         REQUIRED MEMORY:  DEPENDS ON THE MAXIMUM PROBLEM SIZE
C                           (IF THE VALUE OF THE PARAMETERS:
C                           NGUMAX=100, NDETMX=50, NSSMAX=50, NGEMAX=99,
C                           NLIMAX=1400, THEN ABOUT 330 kB FREE MEMORY
C                           IS NECESSARY)
C         OPERATING SYSTEM: IBM PERSONAL COMPUTER DOS 3.30
C         COMPILER:         IBM PROFESSIONAL FORTRAN 1.00
C                               (MATH COPROCESSOR REQUIRED)
C                           OR: RYAN-McFARLAND FORTRAN 2.42
C                               (MATH COPROCESSOR REQUIRED)
C                           OR: MICROSOFT FORTRAN 5.00
C         WORD LENGTH:      32 BITS (BOTH THE REAL AND INTEGER TYPE)
C
C     THE COMPUTER-SPECIFIC STATEMENTS ARE MARKED LIKE THE NEXT LINES:
C
C     C***** ANY EXPRESSION *****
C            ANY FORTRAN STATEMENT
C            ANY FORTRAN STATEMENT
C     C***** ANY EXPRESSION *****
C
C     REFERENCES
C     ==========
C
C     - P.F. ROSE, C.L. DUNFORD: ENDF-6 FORMATS MANUAL.
C       REPORT IAEA-NDS-76, REV. 2. MARCH 1990.
C     - NUCLEAR DATA FOR RADIATION DAMAGE ASSESSMENT AND RELATED SAFETY
C       ASPECTS 1989. REPORT IAEA-TECDOC-572. OCT. 1990.
C       (SEE PAGES 133-137, 139-144, 187-189)
C     - H.J. NOLTHENIUS: COMPUTER PROGRAMS FOR CALCULATING GROUP CROSS-
C       SECTION COVARIANCES FROM UNCERTAINTY PARAMETERS IN THE ENDF/B-V
C       FORMAT: UNC 32/33. REPORT ECN-202, OCT. 1987.
C
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INCLUDE 'X333PARM.FOR'
C***** DOUBLE *****
      DOUBLE PRECISION REUSER,FLUSER
C***** DOUBLE *****
C***** SINGLE *****
C     REAL REUSER,FLUSER
C***** SINGLE *****
      REAL ANII,ANIJ,ANIS,CMUSER,COVI1,COVI2,COVJ1,COVJ2,COVLS1,COVLS2,
     1   COVLST,CRUSER,E,E1,E2,ESAND,EUSER,FFIJN,FLSAND,GEOLI1,GEOLI2,
     1   GEOLJ1,GEOLJ2,GEOLST,GEOMT1,GEOMT2,WEI,WI,WJ,WORK,WORKM,X,
     3   ZA,ZERO
      INTEGER I,IA,IER,II,IPASS,IPOINT,IZ,J,JER,K,K1,KOVFLG,L4,
     1   L613,LTY,MAT,MATLST,MP,MR,MT,MTLIST,N1STU,NDBLK,
     2   NDETEC,NEC1,NEIN,NER1,NFIRST,NGUSE1,NGUSER,
     3   NL,NOTUSD,NP,NPNP,NPOI,NR,NRC,NREC,NRREC,NSSREF
      CHARACTER*80 TITCRD,SPECID
      CHARACTER*8 NAME,FOLNAM,FNAMI,FNAMJ
      CHARACTER*3 CHNGUS
      CHARACTER*2 COVNAM,TWOBL,CRLF,CONI,CONJ
      CHARACTER*1 EQAT,EQATI,EQATJ,ONEBL,TITCR1
      LOGICAL EXDIR,EXXSL,INF,PRD,NEW1,NEW2,NEW3,BUO
      COMMON /CENDF/ E1,E2,E(641),FFIJN(NGEMAX,NGEMAX),LTY,NP,MP
      COMMON /CSAND/ ESAND(641),FLSAND(640),WI(640),WJ(641)
      COMMON /CUSER/ NGUSER,CMUSER(NGUMAX,NGUMAX),EUSER(NGUMX1),
     1   REUSER(NGUMAX),CRUSER(NGUMAX),FLUSER(NGUMAX)
      COMMON /DETECT/ NDETEC,MATLST(0:NDETMX),MTLIST(0:NDETMX),
     1   GEOLST(2,0:NDETMX),COVLST(2,0:NDETMX),ANIS(0:NDETMX),
     2   NRREC(3,0:NDETMX)
      COMMON /DETECH/ FOLNAM(0:NDETMX),COVNAM(0:NDETMX)
      COMMON /SUBSUB/ NPNP,NPOI(3,NSSMAX)
      DIMENSION NAME(0:NDETMX),WORK(NGUMAX,5),WORKM(NGUMAX,NGUMAX),
     1   TITCR1(80)
      EQUIVALENCE (TITCRD,TITCR1(1))
      DATA ZERO/0./,TWOBL/'  '/,ONEBL/' '/
      CRLF=CHAR(13)//CHAR(10)
C-=-=-
C     I/O ORGANIZATION
C-=-=-
      CALL HEAD(' ',0)
      CALL CNCTIO(INF,PRD,L613,TITCRD,NR,NGUSER,EXDIR,EXXSL,L4,IER)
      CALL HEAD(TITCRD,6)
      IF (IER.GT.0) GO TO 70
      WRITE (6,1) NGUSER
    1 FORMAT (/' NUMBER OF THE USER''S ENERGY GROUPS:',I5)
      NGUSE1=NGUSER+1
      CALL ETAP
      CALL READER(L613,NR,NGUSER,NGUSE1,NAME,WORK(1,1),WORK(1,2),
     1   WORK(1,3),EUSER,WORK(1,4),WORK(1,5),WORKM,SPECID,FLSAND,
     2   IER,BUO)
      IF (IER.GT.22) GO TO 65
      IF (EXXSL) THEN
         READ (4,REC=4) NDBLK,NOTUSD
      ELSE
         WRITE (CHNGUS,82) NGUSER
         L4=MIN(L4,80)
         WRITE (4,REC=1) CHNGUS,(ONEBL,I=6,L4),CRLF
         WRITE (4,REC=2) (TITCR1(I),I=1,L4)
         WRITE (4,REC=3) (EUSER(I),I=1,NGUSE1)
         NDBLK=0
         NOTUSD=5
         WRITE (4,REC=4) NDBLK,NOTUSD
      END IF
C-=-=-
C     WORKING UNITS: eV, b
C-=-=-
      DO 2 I=1,NGUSE1
      EUSER(I)=EUSER(I)*1E6
    2 CONTINUE
      E1=MAX(EUSER(1),ESAND(1))
      E2=MIN(EUSER(NGUSER+1),ESAND(641))
      IF (.NOT.BUO) THEN
         DO 3 I=1,640
         FLSAND(I)=FLSAND(I)*(ESAND(I+1)-ESAND(I))*1.E-6
    3    CONTINUE
      END IF
C-=-=-
C     ECHO THE WEIGHTING SPECTRUM, COLLAPSE IT INTO THE USER'S GRID
C-=-=-
      WRITE (6,5) SPECID
      DO 4 I=1,640,12
      II=I+11
      IF (II.GT.640) II=640
      WRITE (6,7) I,(FLSAND(J),J=I,II)
    4 CONTINUE
    5 FORMAT (' '//1X,46('='),' WEIGHTING SPECTRUM (GROUP FLUXES) ',
     1   46('=')/1X,A/1X,127('='))
    7 FORMAT (I5,':  ',1P,12E10.3)
      CALL CALCUF
      IF (INF) CALL INFORM(0,0,0,TITCRD)
C-=-=-
C     LOOP FOR SCANNING THE USER'S INPUT
C     (PASS #1: SELECTING COVERS, 2: DETECTORS, 3: RESPONSE FUNCTIONS)
C-=-=-
      DO 60 IPASS=1,3
      DO 55 IPOINT=1,NDETEC
      CALL MATSEL(IPASS,IPOINT,NEW1,NEW2)
      CALL LUKXSL(IPOINT,NEW3,NREC)
      IF (.NOT.NEW3) GO TO 55
      JER=0
      IF (NEW1) THEN
         CALL INFORM(IPASS,IPOINT,1,' ')
         CALL GENXS(IPASS,IPOINT,JER)
         IER=MAX(IER,JER)
         IF (JER.GT.0) GO TO 55
         IF (IPASS.EQ.3) CALL WXSLWC(IPOINT,NFIRST,N1STU)
         CALL INFORM(IPASS,IPOINT,3,' ')
      END IF
      IF (.NOT.NEW2) GO TO 55
      CALL INFORM(IPASS,IPOINT,2,' ')
      MR=NRREC(2,IPOINT)
      IF (MR.LT.0) THEN
         IF (IPASS.EQ.2) THEN
            WRITE (6,90) IPOINT,FOLNAM(IPOINT),GEOLST(1,IPOINT),
     1      GEOLST(2,IPOINT),COVNAM(IPOINT),COVLST(1,IPOINT),
     2      COVLST(2,IPOINT),ANIS(IPOINT),MATLST(IPOINT),MTLIST(IPOINT)
            CALL WXSLWC(IPOINT,NFIRST,N1STU)
            IER=MAX(IER,32)
            CALL ERRLOG(32)
         END IF
         GO TO 55
      END IF
      READ (3,80,REC=MR) ZA,NL,MAT,MT
      ZA=ZA/1000.
      X=AMOD(ZA,1.)
      IZ=NINT(ZA-X)
      IA=NINT(X*1000.)
      CALL XSCLOG(MAT,MT,IZ,IA)
      E(1)=ESAND(1)
      E(2)=ESAND(641)
      DO 16 I=1,NGUSER
      DO 15 J=1,NGUSER
      CMUSER(I,J)=0.
   15 CONTINUE
   16 CONTINUE
      NRC=NRC+1
C-=-=-
C     CHECK IF THERE ARE REFERRED COVARIANCE MATRICES IN THE SUB-SUBSECS
C-=-=-
C     * * * * * * * * * * * * * W A R N I N G * * * * * * * * * * * *
C
C     THE VERSION JAN 29, 1991 OF THE IRDF90G LIBRARY DOES NOT CONTAIN
C     CROSS SECTIONS HAVING NC-TYPE SUB-SUBSECTIONS, THEREFORE THIS
C     PART OF THE PROGRAM CANNOT BE TESTED
C-=-=-
      CALL SECMAP(IPOINT,NSSREF)
   20 IF (NSSREF.GT.0) THEN
         JER=0
         CALL SCTION(L613,0,NPOI(3,NSSREF),N1STU,KOVFLG,JER)
         IER=MAX(IER,JER)
         IF (JER.GT.0) GO TO 55
         CALL GENNAM(NPOI(1,NSSREF),NPOI(2,NSSREF),NAME)
         READ (4,REC=4) NDBLK,NOTUSD
         WRITE (4,REC=NRC) NAME,ZERO,ZERO,TWOBL,ZERO,ZERO,ZERO,
     1      MAT,MT,N1STU,KOVFLG
         NRC=NRC+1
         IF (KOVFLG.LT.2) THEN
            WRITE (4,REC=NRC) (CRUSER(J),J=N1STU,NGUSER)
            NRC=NRC+1
         ELSE
C***** TEMPORARY, TO BYPASS WRITING INVALID CROSS-COVARIANCE MATRICES
            WEI=0.
C***** TEMPORARY, TO BYPASS WRITING INVALID CROSS-COVARIANCE MATRICES
            WRITE (4,REC=NRC) NAME,ZERO,ZERO,TWOBL,ZERO,ZERO,
     1      MAT,MT,WEI
            NRC=NRC+1
         END IF
         IF (KOVFLG.EQ.1.OR.(KOVFLG.EQ.2.AND.WEI.EQ.1.)) THEN
            DO 25 I=N1STU,NGUSER
            WRITE (4,REC=NRC+I-N1STU) (CMUSER(I,J),J=N1STU,NGUSER)
   25       CONTINUE
            J=NGUSER-N1STU+1
            NRC=NRC+J
            NOTUSD=NOTUSD+J
         END IF
         NDBLK=NDBLK+1
         NOTUSD=NOTUSD+2
         WRITE (4,REC=4) NDBLK,NOTUSD
         NSSREF=NSSREF-1
         GO TO 20
      END IF
C-=-=-
C     PROCESSING ONLY NI SUB-SUBSECTIONS
C-=-=-
      JER=0
      CALL SCTION(L613,IPOINT,NRREC(2,IPOINT),N1STU,KOVFLG,JER)
      IER=MAX(IER,JER)
      IF (JER.GT.0) GO TO 55
      GEOMT1=GEOLST(1,IPOINT)
      IF (GEOMT1.GE.0) THEN
         EQAT='='
      ELSE
         EQAT='@'
         GEOMT1=-GEOMT1
      END IF
      GEOMT2=GEOLST(2,IPOINT)
      COVLS1=COVLST(1,IPOINT)
      COVLS2=COVLST(2,IPOINT)
      WRITE (6,87) FOLNAM(IPOINT),EQAT,GEOMT1,GEOMT2,
     1   COVNAM(IPOINT),COVLS1,COVLS2,ANIS(IPOINT)
      READ (4,REC=4) NDBLK,NOTUSD
      NRC=NOTUSD
      GEOMT1=GEOLST(1,IPOINT)
      WRITE (4,REC=NRC) FOLNAM(IPOINT),GEOMT1,GEOMT2,COVNAM(IPOINT),
     1   COVLS1,COVLS2,ANIS(IPOINT),MAT,MT,N1STU,KOVFLG
      NRC=NRC+1
      WRITE (4,REC=NRC) (CRUSER(J),J=N1STU,NGUSER)
      NRC=NRC+1
      IF (KOVFLG.GT.0) THEN
         DO 30 I=N1STU,NGUSER
         WRITE (4,REC=NRC+I-N1STU) (CMUSER(I,J),J=N1STU,NGUSER)
   30    CONTINUE
         J=NGUSER-N1STU+1
         NRC=NRC+J
         NOTUSD=NOTUSD+J
      END IF
      NDBLK=NDBLK+1
      NOTUSD=NOTUSD+2
      WRITE (4,REC=4) NDBLK,NOTUSD
      JER=0
      CALL PRLTCM(N1STU,NGUSER,JER)
      IER=MAX(IER,JER)
      JER=0
      ENDFILE (12)
      BACKSPACE (12)
      REWIND (12)
   40 READ (12,END=50) I,FNAMI,GEOLI1,GEOLI2,CONI,COVI1,COVI2,ANII
      READ (12) J,FNAMJ,GEOLJ1,GEOLJ2,CONJ,COVJ1,COVJ2,ANIJ
      IF (JER.EQ.0) THEN
         WRITE (L613,100)
         JER=1
      END IF
      EQATI='='
      EQATJ='='
      IF (GEOLI1.LT.0.) THEN
         EQATI='@'
         GEOLI1=-GEOLI1
      END IF
      IF (GEOLJ1.LT.0.) THEN
         EQATJ='@'
         GEOLJ1=-GEOLJ1
      END IF
      WRITE (L613,101) FNAMI,EQATI,GEOLI1,GEOLI2,CONI,COVI1,COVI2,ANII,
     1                 FNAMJ,EQATJ,GEOLJ1,GEOLJ2,CONJ,COVJ1,COVJ2,ANIJ
      READ (12) NER1,NEC1
      NEIN=NER1+NEC1+2
      READ (12) (E(K),K=1,NEIN)
      DO 45 K=1,NER1
      READ (12) (FFIJN(K,K1),K1=1,NEC1)
   45 CONTINUE
      CALL STRUC6(I,J,NER1,NEC1)
      GO TO 40
   50 REWIND (12)
      CALL INFORM(IPASS,IPOINT,3,' ')
   55 CONTINUE
   60 CONTINUE
   65 NRC=NRC-1
      CALL XSLIST
      IF (PRD) THEN
         CALL HEAD(TITCRD,0)
      ELSE
         CALL HEAD(TITCRD,-1)
      END IF
      CALL HEAD(TITCRD,6)
      IF (EXDIR) CLOSE (UNIT=2,STATUS='KEEP')
      CLOSE (UNIT=3,STATUS='KEEP')
      CLOSE (UNIT=4,STATUS='KEEP')
      CLOSE (UNIT=8,STATUS='KEEP')
      IF (L613.EQ.13) THEN
         CLOSE (UNIT=9,STATUS='DELETE')
         CLOSE (UNIT=10,STATUS='DELETE')
         CLOSE (UNIT=11,STATUS='DELETE')
         CLOSE (UNIT=12,STATUS='DELETE')
      ELSE
         CLOSE (UNIT=9,STATUS='KEEP')
         CLOSE (UNIT=10,STATUS='KEEP')
         CLOSE (UNIT=11,STATUS='KEEP')
         CLOSE (UNIT=12,STATUS='KEEP')
      END IF
   70 CALL ERXPLN(IER)
   80 FORMAT (F11.0,44X,I11,I4,2X,I3)
   82 FORMAT (I3)
   87 FORMAT (////1X,127('=')/
     1   ' EVALUATED CROSS SECTION OF THE DETECTOR:  ',A8,2X,A1,
     2   1P,2E11.3,2X,A2,1P,2E11.3,0P,F7.1/1X,127('='))
   90 FORMAT (' '/' USER''S REQUEST LIST ITEM No =',I3//
     1   ' NAME:',T20,A/' DETSIZ:',T20,1P,2E11.3/' COVNAM:',T20,A/
     2   ' COVTHC:',T20,1P,2E11.3/' ANISOT:',T20,1P,E11.3//
     3   ' FILE MF=33 IS LACKING, MAT =',I5,' MF =',I3//
     4   ' ??? 32 ???   PROCESSING SKIPPED')
  100 FORMAT (//' CROSS-COVARIANCE MATRIX CALCULATION FOLLOWS FOR:'/)
  101 FORMAT (1X,A8,2X,A1,1P,2E10.3,2X,A2,1P,2E10.3,0P,F8.2,'   AND'/
     1        1X,A8,2X,A1,1P,2E10.3,2X,A2,1P,2E10.3,0P,F8.2/)
      END
