      SUBROUTINE FNDNDF(REANAM,I3,I33,ICOV,MATFND,MTFND)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS SUBROUTINE LOOKS FOR THE REACTION CROSS SECTION IDENTIFIED
C     BY THE MCELROY'S CONVENTION (REANAM) IN THE FILES MF=3 AND MF=33
C     OF THE IRDFxxG CROSS SECTION LIBRARY (FORMATTED, ENDF-6)
C     AND RETURNS THE VALUES OF THE "REC=" PARAMETER OF THE DIRECT
C     ACCESS READ STATEMENT IN I3, I33, AND ICOV.
C
C     REANAM = 'CHEMICAL_SYMBOL'[//'MASS_NUMBER'][//'REACTION_CODE']
C              MAX. LENGTH = 8 CHARACTERS
C              OMITTED OR '0' MASS_NUMBER MEANS NATURAL ELEMENT
C
C     INTERPRETED VALUES OF REACTION_CODE:                 EXAMPLE:
C        '2'     FOR THE REACTION (N,2N)         (MT=16)    MN552
C        'F'     FOR THE REACTION (N,F)          (MT=18)    TH232F
C        'NP'    FOR THE REACTION (N,NP)         (MT=28)    TI46NP
C        'N'     FOR THE REACTION (N,N')         (MT=51)    IN115N
C        'G'     FOR THE REACTION (N,GAMMA)      (MT=102)   AU197G
C        'P'     FOR THE REACTION (N,P)          (MT=103)   AL27P
C        'T'     FOR THE REACTION (N,T)          (MT=105)   LI6T
C        'A'     FOR THE REACTION (N,ALPHA)      (MT=107)   LI6A
C        '-H'    FOR HYDROGEN PRODUCTION         (MT=203)   C-H
C        '-D'    FOR DEUTERIUM PRODUCTION        (MT=204)   LI7-D
C        '-T'    FOR TRITIUM PRODUCTION          (MT=205)   LI6-T
C        '-HE3'  FOR HELIUM-3 PRODUCTION         (MT=206)   CR24-HE3
C        '-HE'   FOR HELIUM PRODUCTION           (MT=207)   LI6-HE
C
C     THE VALUES OF THE OUTPUT PARAMETERS DEPEND ON THE REANAM:
C     - IF THE REACTION_CODE IS PRESENT, THEN THE PROGRAM LOOKS FOR
C       THE GIVEN REACTION IN THE FILES MF=3, AND 33. (I3 AND I33
C       POINT TO THE FOUND RECORDS IN THE FILES MF=3 AND 33,
C       RESPECTIVELY, ICOV REMAINS UNALTERED)
C     - IF ONLY THE CHEMICAL SYMBOL IS GIVEN IN REACTION_NAME, THEN
C       THE PROGRAM LOOKS FOR THE TOTAL CROSS SECTION OF THE GIVEN
C       "COVER" MATERIAL (MF=3, MT=1).
C       (ICOV POINTS TO THE FOUND RECORD, I3 AND I33 REAMAIN UNALTERED)
C     IF THE REQUESTED CROSS SECTION CANNOT BE FOUND (UNACCESSABLE
C     LIBRARY, ERRONEOUS REANAM, MISSING DATA SECTION IN LIBRARY,
C     &C), THE PERTINENT PARAMETERS ARE SET TO -1.
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      REAL ZA,ZALIB
      INTEGER I,I3,I33,ICOV,IER,IERH,L,MAT,MATFND,MF,MT,MTDEXP,MTFND,
     1   NR,NS
      LOGICAL FIND,COVER,HYPHEN
      CHARACTER*(*) REANAM
      CHARACTER*8 NAME,LEFT,RIGHT
      SAVE
      NAME=REANAM
      CALL UPPCAS(NAME,8)
      CALL LASTCH(NAME,L)
      DO 27 I=1,L
      HYPHEN=NAME(I:I).EQ.'-'
      IF (HYPHEN) THEN
         IERH=1
         LEFT=NAME(1:I)
         RIGHT=NAME(I+1:L)
         CALL NAMEZA(LEFT,ZA,MTFND,IER)
         MTFND=MTDEXP(RIGHT)
         GO TO 28
      END IF
   27 CONTINUE
      IERH=0
      CALL NAMEZA(NAME,ZA,MTFND,IER)
   28 IF (IER.GT.IERH) GO TO 40
      COVER=MTFND.EQ.1
      REWIND (10)
   30 READ (10,END=40) NR,MAT,MF,MT
      IF (MT.EQ.451) THEN
         READ (3,32,REC=NR) ZALIB,MAT,MF,MT,NS
   32    FORMAT (F11.0,55X,I4,I2,I3,I5)
         FIND=ABS(ZA-ZALIB).LT..001
         MATFND=MAT
         IF (FIND) THEN
   35       READ (10,END=40) NR,MAT,MF,MT
            IF (MAT.NE.MATFND) GO TO 40
            IF (MT.NE.MTFND) GO TO 35
            IF (COVER) THEN
               IF (MF.EQ.3) ICOV=NR
               GO TO 40
            ELSE
               IF (MF.EQ.3) THEN
                  I3=NR
               ELSE IF (MF.EQ.33) THEN
                  I33=NR
               END IF
               GO TO 35
            END IF
         END IF
      END IF
      GO TO 30
   40 RETURN
      END
      SUBROUTINE IPRGEO(MAT,MT,GEOM,COVN,COVT,DSD,DX,CSD,CX,IER)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS SUBROUTINE INTERPRETES THE GEOMETRY DATA
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      REAL ABUD,AG,AWR,COVT,CSD,CX,DE,DENSTY,DSD,DX,GEOM,ZA
      INTEGER IA,IER,IZ,JZ,LUKREF,MAC,MAT,MT,NR
      CHARACTER*2 COVN
      DIMENSION GEOM(2),COVT(2)
      SAVE
C-=-=-
C     DETECTOR
C-=-=-
      AG=ABS(GEOM(1))
      IF (AG.EQ.0.) THEN
         DSD=0.
         DX=0.
      ELSE
         NR=LUKREF(MAT,3,MT)
         IF (NR.LT.0) THEN
            WRITE (6,81) MAT,MT
            RETURN
         END IF
         READ (3,80,REC=NR) ZA
         JZ=MAT/100
         IA=NINT(MOD(ZA,1000.))
         CALL NZCURV(JZ,IA,AWR,ABUD)
         DE=GEOM(2)
         IF (DE.EQ.0.) THEN
            DE=DENSTY(JZ)
            IF (DE.LT.0.) THEN
               WRITE (6,82) JZ,MAT,MT
               WRITE (6,85)
               IER=14
               CALL ERRLOG(14)
               RETURN
            END IF
         END IF
         IF (AG.LE.999.999) THEN
            DX=AG
            IF (DE.LE.999.999) THEN
               DSD=DE*ABUD/AWR*.6023*DX
            ELSE
               DSD=DE*1E-24
            END IF
         ELSE
            IF (DE.LE.999.999) DE=DE*ABUD/AWR*.6023E24
            DX=AG/DE
            DSD=AG*1E-24
         END IF
      END IF
C-=-=-
C     COVER
C-=-=-
      AG=ABS(COVT(1))
      IF (AG.EQ.0.) THEN
         CSD=0.
         CX=0.
      ELSE
         JZ=IZ(COVN)
         MAC=JZ*100
         NR=LUKREF(MAC,3,1)
         IF (NR.LT.0) THEN
            WRITE (6,83) COVN,MAC
            RETURN
         END IF
         READ (3,80,REC=NR) ZA,AWR
         DE=COVT(2)
         IF (DE.EQ.0.) THEN
            DE=DENSTY(JZ)
            IF (DE.LT.0.) THEN
               WRITE (6,84) COVN,JZ
               WRITE (6,85)
               IER=15
               CALL ERRLOG(15)
               RETURN
            END IF
         END IF
         IF (AG.LE.999.999) THEN
            CX=AG
            IF (DE.LE.999.999) THEN
               CSD=DE/AWR*.6023*CX
            ELSE
               CSD=DE*1E-24
            END IF
         ELSE
            IF (DE.LE.999.999) DE=DE/AWR*.6023E24
            CX=AG/DE
            CSD=AG*1E-24
         END IF
      END IF
      RETURN
   80 FORMAT (2F11.0)
   81 FORMAT (' '//
     1   ' ??? 31 ??? THE FILE MF=3 IS LACKING, MAT =',I5,' MT =',I3)
   82 FORMAT (' '//' ??? 14 ??? THE DETECTOR: Z=',I3,
     1   ' MAT =',I5,' MT =',I3)
   83 FORMAT (' '//' ??? 31 ??? FOR THE COVER MATERIAL: ',A,
     1   ' THE FILE MF=3 IS LACKING, MAT =',I5,' MT =  1')
   84 FORMAT (' '//' ??? 15 ??? THE COVER MATERIAL: ',A,' MAT =',I5)
   85 FORMAT (' IS NOT STABLE. COMPLETE GEOMETRY DESCRIPTION NEEDED.'/
     1   ' (REQUEST IGNORED.)')
      END
      FUNCTION IZ(CH)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS FUNCTION CALCULATES THE IZ ATOMIC NUMBER TO THE
C     CH CHEMICAL SYMBOL
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INTEGER I,IZ
      CHARACTER*2 CH
      CALL CHESYM(1,CH,I)
      IZ=I
      RETURN
      END
      SUBROUTINE LUKRFD(IWHAT,MAT1,MT1,NREF,IREF)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS SUBROUTINE LOOKS FOR THE REFERRED DETECTOR SET MAT1,MT1
C     IN THE USER'S INPUT
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INCLUDE 'X333PARM.FOR'
      REAL ANIS,COVLST,GEOLST
      INTEGER IREF,IWHAT,J,MAT1,MATLST,MT1,MTLIST,N1,NDETEC,
     1   NREF,NRREC
      CHARACTER*8 FOLNAM
      CHARACTER*2 COVNAM
      COMMON /DETECT/ NDETEC,MATLST(0:NDETMX),MTLIST(0:NDETMX),
     1   GEOLST(2,0:NDETMX),COVLST(2,0:NDETMX),ANIS(0:NDETMX),
     2   NRREC(3,0:NDETMX)
      COMMON /DETECH/ FOLNAM(0:NDETMX),COVNAM(0:NDETMX)
      DIMENSION IREF(*)
      NREF=0
      DO 10 J=1,IWHAT-1
      IF (MATLST(J).NE.MAT1) GO TO 10
      IF (MTLIST(J).NE.MT1) GO TO 10
      CALL LUKSXS(2,J,1,N1,.TRUE.)
      IF (N1.GT.0) THEN
         NREF=NREF+1
         IREF(NREF)=J
      END IF
    5 CONTINUE
   10 CONTINUE
      RETURN
      END
      SUBROUTINE NXTWRD(STRING,I1ST,LNGTH)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS SUBROUTINE LOOKS FOR THE NEXT WORD IN THE SUBSTRING
C     STRING(I1ST:80); THE FOUND WORD IS: STRING(N1ST:I1ST+LNGTH-1)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INTEGER I,I1ST,LNGTH
      CHARACTER*80 STRING
      DO 1 I=I1ST,80
      IF (STRING(I:I).NE.' ') GO TO 2
    1 CONTINUE
      LNGTH=0
      RETURN
    2 I1ST=I
      DO 3 I=I1ST,80
      IF (STRING(I:I).EQ.' ') GO TO 4
    3 CONTINUE
      I=81
    4 LNGTH=I-I1ST
      RETURN
      END
      SUBROUTINE REACOD(IWHAT,R,M)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS SUBROUTINE CONVERTS MT TO/FROM MCELROY'S DOSIMETRY
C     REACTION CODE
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INTEGER I,IWHAT,KODES,M
      CHARACTER*1 R,CODES
      DIMENSION CODES(7),KODES(7)
      SAVE
      DATA CODES /'2','A','F','G','N','P','T'/
      DATA KODES / 16,107, 18,102, 51,103,105/
      IF (IWHAT.EQ.1) THEN
         DO 1 I=1,7
         IF (R.EQ.CODES(I)) THEN
            M=KODES(I)
            RETURN
         END IF
    1    CONTINUE
         M=-1
      ELSE
         DO 2 I=1,7
         IF (M.EQ.KODES(I)) THEN
            R=CODES(I)
            RETURN
         END IF
    2    CONTINUE
         R='?'
      END IF
      RETURN
      END
      SUBROUTINE UPPCAS(STRING,L)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THIS SUBROUTINE CONVERTS THE LOWER CASE LETTERS TO UPPER CASE
C     LETTERS, AND REMOVES THE COMMAS AND APOSTROPHES
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      INTEGER I,IC,L
      CHARACTER*(*) STRING
      CHARACTER*1 C
      DO 1 I=1,L
      C=STRING(I:I)
      IF (C.EQ.','.OR.C.EQ.'''') THEN
         STRING(I:I)=' '
      ELSE
         IC=ICHAR(C)
         IF (IC.GE.97.AND.IC.LE.122) THEN
            IC=IC-32
            STRING(I:I)=CHAR(IC)
         END IF
      END IF
    1 CONTINUE
      RETURN
      END
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     DOS TIME- ADN DATE-FUNCTION-CALLING SUBROUTINES FOR THE LAHEY
C     F77L-EM/32 VER. 4.01 COMPILER
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
C     THE SUBROUTINE GETTIM AND GETDAT ARE LIBRARY ROUTINES OF THE
C     RYAN-MCFARLAND  AND MICROSOFT COMPILERS, THEREFORE USING THEM
C     COMMENT OUT THE  WHOLE TEXT BELOW (INSERT THE COMMENT SIGN 'C'-S
C     INTO THE FIRST COLUMN OF THE REMAINDER PART OF THE FILE)
C-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
      SUBROUTINE GETTIM(I,J,K,L)
      INTEGER*2 I,J,K,L
      CHARACTER*11 RESULT
      CALL TIME(RESULT)
      READ (RESULT,1) I,J,K,L
      RETURN
    1 FORMAT (3(I2,1X),I2)
      END
      SUBROUTINE GETDAT(I,J,K)
      INTEGER*2 I,J,K
      CHARACTER*8 RESULT
      CALL DATE(RESULT)
      READ (RESULT,1) J,K,I
      I=I+1900
      RETURN
    1 FORMAT (2(I2,1X),I2)
      END
